@@ -105,7 +105,6 @@ class Service(asyncio.AbstractServer):
     def __init__(self, loop, proto):
         self._loop = loop
         self._proto = proto
-        self._closing = False
 
     @property
     def transport(self):
@@ -120,9 +119,9 @@ class Service(asyncio.AbstractServer):
         return transport
 
     def close(self):
-        if self._closing:
+        if self._proto.closing:
             return
-        self._closing = True
+        self._proto.closing = True
         if self._proto.transport is None:
             return
         self._proto.transport.close()
@@ -143,6 +142,8 @@ class _BaseProtocol(interface.ZmqProtocol):
         self.transport = None
         self.done_waiters = []
         self.packer = _Packer(translation_table=translation_table)
+        self.pending_waiters = set()
+        self.closing = False
 
     def connection_made(self, transport):
         self.transport = transport
@@ -152,6 +153,12 @@ class _BaseProtocol(interface.ZmqProtocol):
         for waiter in self.done_waiters:
             waiter.set_result(None)
 
+    def add_pending(self, fut):
+        self.pending_waiters.add(fut)
+
+    def discard_pending(self, fut):
+        self.pending_waiters.discard(fut)
+
 
 class _BaseServerProtocol(_BaseProtocol):
 
@@ -164,7 +171,6 @@ class _BaseServerProtocol(_BaseProtocol):
         self.handler = handler
         self.log_exceptions = log_exceptions
         self.exclude_log_exceptions = exclude_log_exceptions
-        self.pending_waiters = set()
 
     def connection_lost(self, exc):
         super().connection_lost(exc)
